@@ -21,9 +21,7 @@
  *
  **************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include <string.h>
 #include <sys/types.h>
@@ -33,19 +31,12 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include <glib/gi18n-lib.h>
-#include <nm-setting-connection.h>
-#include <nm-setting-8021x.h>
-#include <nm-utils.h>
+#include "nm-default.h"
 
 #include "auth-helpers.h"
 #include "nm-openvpn.h"
-#include "src/nm-openvpn-service.h"
-#include "common/utils.h"
-
-#define PW_TYPE_SAVE   0
-#define PW_TYPE_ASK    1
-#define PW_TYPE_UNUSED 2
+#include "nm-openvpn-service-defines.h"
+#include "utils.h"
 
 #define BLOCK_HANDLER_ID "block-handler-id"
 
@@ -58,10 +49,9 @@ show_password (GtkToggleButton *togglebutton, GtkEntry *password_entry)
 static GtkWidget *
 setup_secret_widget (GtkBuilder *builder,
                      const char *widget_name,
-                     NMSettingVPN *s_vpn,
+                     NMSettingVpn *s_vpn,
                      const char *secret_key)
 {
-	NMSettingSecretFlags pw_flags = NM_SETTING_SECRET_FLAG_NONE;
 	GtkWidget *widget;
 	GtkWidget *show_passwords;
 	const char *tmp;
@@ -76,19 +66,30 @@ setup_secret_widget (GtkBuilder *builder,
 		tmp = nm_setting_vpn_get_secret (s_vpn, secret_key);
 		if (tmp)
 			gtk_entry_set_text (GTK_ENTRY (widget), tmp);
-
-		nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_key, &pw_flags, NULL);
-		g_object_set_data (G_OBJECT (widget), "flags", GUINT_TO_POINTER (pw_flags));
 	}
 
 	return widget;
 }
 
+typedef struct {
+	GtkWidget *widget1;
+	GtkWidget *widget2;
+} TlsChooserSignalData;
+
 static void
-tls_cert_changed_cb (GtkWidget *widget, GtkWidget *next_widget)
+tls_chooser_signal_data_destroy (gpointer data, GClosure *closure)
 {
-	GtkFileChooser *this, *next;
-	char *fname, *next_fname;
+	g_slice_free (TlsChooserSignalData, data);
+}
+
+static void
+tls_cert_changed_cb (GtkWidget *widget, gpointer data)
+{
+	GtkWidget *other_widgets[2] = { ((TlsChooserSignalData *) data)->widget1,
+	                                ((TlsChooserSignalData *) data)->widget2 };
+	GtkFileChooser *this, *others[2];
+	char *fname, *other_fnames[2];
+	int i;
 
 	/* If the just-changed file chooser is a PKCS#12 file, then all of the
 	 * TLS filechoosers have to be PKCS#12.  But if it just changed to something
@@ -100,42 +101,48 @@ tls_cert_changed_cb (GtkWidget *widget, GtkWidget *next_widget)
 	 */
 
 	this = GTK_FILE_CHOOSER (widget);
-	next = GTK_FILE_CHOOSER (next_widget);
+	others[0] = GTK_FILE_CHOOSER (other_widgets[0]);
+	others[1] = GTK_FILE_CHOOSER (other_widgets[1]);
 
 	fname = gtk_file_chooser_get_filename (this);
+	other_fnames[0] = gtk_file_chooser_get_filename (others[0]);
+	other_fnames[1] = gtk_file_chooser_get_filename (others[1]);
+
 	if (is_pkcs12 (fname)) {
 		/* Make sure all choosers have this PKCS#12 file */
-		next_fname = gtk_file_chooser_get_filename (next);
-		if (!next_fname || strcmp (fname, next_fname)) {
-			/* Next chooser was different, make it the same as the first */
-			gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (next_widget), BLOCK_HANDLER_ID));
-			g_signal_handler_block (G_OBJECT (next_widget), id);
-			gtk_file_chooser_set_filename (next, fname);
-			g_signal_handler_unblock (G_OBJECT (next_widget), id);
+		for (i = 0; i < 2; i++) {
+			if (!other_fnames[i] || strcmp (fname, other_fnames[i])) {
+				/* Next chooser was different, make it the same as the first */
+				gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (other_widgets[i]),
+				                                                 BLOCK_HANDLER_ID));
+				g_signal_handler_block (other_widgets[i], id);
+				gtk_file_chooser_set_filename (others[i], fname);
+				g_signal_handler_unblock (other_widgets[i], id);
+			}
+		}
+	} else {
+		/* Just-chosen file isn't PKCS#12 or no file was chosen, so clear out other
+		 * file selectors that have PKCS#12 files in them.
+		 */
+		for (i = 0; i < 2; i++) {
+			if (is_pkcs12 (other_fnames[i])) {
+				gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (other_widgets[i]),
+				                                                 BLOCK_HANDLER_ID));
+				g_signal_handler_block (other_widgets[i], id);
+				gtk_file_chooser_unselect_all (others[i]);
+				g_signal_handler_unblock (other_widgets[i], id);
+			}
 		}
-		g_free (fname);
-		g_free (next_fname);
-		return;
 	}
 	g_free (fname);
-
-	/* Just-chosen file isn't PKCS#12 or no file was chosen, so clear out other
-	 * file selectors that have PKCS#12 files in them.
-	 */
-	next_fname = gtk_file_chooser_get_filename (next);
-	if (is_pkcs12 (next_fname)) {
-		gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (next_widget), BLOCK_HANDLER_ID));
-		g_signal_handler_block (G_OBJECT (next_widget), id);
-		gtk_file_chooser_set_filename (next, NULL);
-		g_signal_handler_unblock (G_OBJECT (next_widget), id);
-	}
-	g_free (next_fname);
+	g_free (other_fnames[0]);
+	g_free (other_fnames[1]);
 }
 
 static void
 tls_setup (GtkBuilder *builder,
            GtkSizeGroup *group,
-           NMSettingVPN *s_vpn,
+           NMSettingVpn *s_vpn,
            const char *prefix,
            GtkWidget *ca_chooser,
            ChangedCallback changed_cb,
@@ -145,6 +152,7 @@ tls_setup (GtkBuilder *builder,
 	const char *value;
 	char *tmp;
 	GtkFileFilter *filter;
+	TlsChooserSignalData *ca_chooser_data, *cert_data, *key_data;
 	gulong id1, id2, id3;
 
 	tmp = g_strdup_printf ("%s_user_cert_chooser", prefix);
@@ -183,10 +191,23 @@ tls_setup (GtkBuilder *builder,
 			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (key), value);
 	}
 
+	ca_chooser_data = g_slice_new0 (TlsChooserSignalData);
+	ca_chooser_data->widget1 = cert;
+	ca_chooser_data->widget2 = key;
+	cert_data = g_slice_new0 (TlsChooserSignalData);
+	cert_data->widget1 = ca_chooser;
+	cert_data->widget2 = key;
+	key_data = g_slice_new0 (TlsChooserSignalData);
+	key_data->widget1 = ca_chooser;
+	key_data->widget2 = cert;
+
 	/* Link choosers to the PKCS#12 changer callback */
-	id1 = g_signal_connect (ca_chooser, "selection-changed", G_CALLBACK (tls_cert_changed_cb), cert);
-	id2 = g_signal_connect (cert, "selection-changed", G_CALLBACK (tls_cert_changed_cb), key);
-	id3 = g_signal_connect (key, "selection-changed", G_CALLBACK (tls_cert_changed_cb), ca_chooser);
+	id1 = g_signal_connect_data (ca_chooser, "selection-changed", G_CALLBACK (tls_cert_changed_cb),
+	                             ca_chooser_data, tls_chooser_signal_data_destroy, 0);
+	id2 = g_signal_connect_data (cert, "selection-changed", G_CALLBACK (tls_cert_changed_cb),
+	                             cert_data, tls_chooser_signal_data_destroy, 0);
+	id3 = g_signal_connect_data (key, "selection-changed", G_CALLBACK (tls_cert_changed_cb),
+	                             key_data, tls_chooser_signal_data_destroy, 0);
 
 	/* Store handler id to be able to block the signal in tls_cert_changed_cb() */
 	g_object_set_data (G_OBJECT (ca_chooser), BLOCK_HANDLER_ID, GSIZE_TO_POINTER (id1));
@@ -199,93 +220,15 @@ tls_setup (GtkBuilder *builder,
 	g_free (tmp);
 	gtk_size_group_add_widget (group, widget);
 	g_signal_connect (widget, "changed", G_CALLBACK (changed_cb), user_data);
-}
 
-static void
-pw_type_combo_changed_cb (GtkWidget *combo, gpointer user_data)
-{
-	GtkWidget *entry = user_data;
-
-	/* If the user chose "Not required", desensitize and clear the correct
-	 * password entry.
-	 */
-	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (combo))) {
-	case PW_TYPE_ASK:
-	case PW_TYPE_UNUSED:
-		gtk_entry_set_text (GTK_ENTRY (entry), "");
-		gtk_widget_set_sensitive (entry, FALSE);
-		break;
-	default:
-		gtk_widget_set_sensitive (entry, TRUE);
-		break;
-	}
-}
-
-static void
-init_one_pw_combo (GtkBuilder *builder,
-                   NMSettingVPN *s_vpn,
-                   const char *prefix,
-                   const char *secret_key,
-                   GtkWidget *entry_widget,
-                   ChangedCallback changed_cb,
-                   gpointer user_data)
-{
-	int active = -1;
-	GtkWidget *widget;
-	GtkListStore *store;
-	GtkTreeIter iter;
-	const char *value = NULL;
-	char *tmp;
-	guint32 default_idx = 1;
-	NMSettingSecretFlags pw_flags = NM_SETTING_SECRET_FLAG_NONE;
-
-	/* If there's already a password and the password type can't be found in
-	 * the VPN settings, default to saving it.  Otherwise, always ask for it.
-	 */
-	value = gtk_entry_get_text (GTK_ENTRY (entry_widget));
-	if (value && strlen (value))
-		default_idx = 0;
-
-	store = gtk_list_store_new (1, G_TYPE_STRING);
-	if (s_vpn)
-		nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_key, &pw_flags, NULL);
-
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Saved"), -1);
-	if (   (active < 0)
-	    && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED)
-	    && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
-		active = PW_TYPE_SAVE;
-	}
-
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Always Ask"), -1);
-	if ((active < 0) && (pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED))
-		active = PW_TYPE_ASK;
-
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, _("Not Required"), -1);
-	if ((active < 0) && (pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED))
-		active = PW_TYPE_UNUSED;
-
-	tmp = g_strdup_printf ("%s_pass_type_combo", prefix);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
-	g_assert (widget);
-	g_free (tmp);
-
-	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
-	g_object_unref (store);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? default_idx : active);
-	pw_type_combo_changed_cb (widget, entry_widget);
-
-	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (pw_type_combo_changed_cb), entry_widget);
-	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
+	nma_utils_setup_password_storage (widget, 0, (NMSetting *) s_vpn, NM_OPENVPN_KEY_CERTPASS,
+	                                  TRUE, FALSE);
 }
 
 static void
 pw_setup (GtkBuilder *builder,
-          GtkSizeGroup *group, 
-          NMSettingVPN *s_vpn,
+          GtkSizeGroup *group,
+          NMSettingVpn *s_vpn,
           const char *prefix,
           ChangedCallback changed_cb,
           gpointer user_data)
@@ -313,13 +256,14 @@ pw_setup (GtkBuilder *builder,
 	gtk_size_group_add_widget (group, widget);
 	g_signal_connect (widget, "changed", G_CALLBACK (changed_cb), user_data);
 
-	init_one_pw_combo (builder, s_vpn, prefix, NM_OPENVPN_KEY_PASSWORD, widget, changed_cb, user_data);
+	nma_utils_setup_password_storage (widget, 0, (NMSetting *) s_vpn, NM_OPENVPN_KEY_PASSWORD,
+	                                  TRUE, FALSE);
 }
 
 void
 tls_pw_init_auth_widget (GtkBuilder *builder,
                          GtkSizeGroup *group,
-                         NMSettingVPN *s_vpn,
+                         NMSettingVpn *s_vpn,
                          const char *contype,
                          const char *prefix,
                          ChangedCallback changed_cb,
@@ -376,7 +320,7 @@ tls_pw_init_auth_widget (GtkBuilder *builder,
 void
 sk_init_auth_widget (GtkBuilder *builder,
                      GtkSizeGroup *group,
-                     NMSettingVPN *s_vpn,
+                     NMSettingVpn *s_vpn,
                      ChangedCallback changed_cb,
                      gpointer user_data)
 {
@@ -441,9 +385,6 @@ sk_init_auth_widget (GtkBuilder *builder,
 	g_object_unref (store);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 0 : active);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "sk_dir_help_label"));
-	gtk_size_group_add_widget (group, widget);
-
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "sk_local_address_entry"));
 	gtk_size_group_add_widget (group, widget);
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (changed_cb), user_data);
@@ -484,15 +425,16 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	char *tmp;
 	gboolean valid, encrypted = FALSE;
 	GtkWidget *widget;
-	char *str;
+	NMSettingSecretFlags pw_flags;
+	gboolean secrets_required = TRUE;
 
 	tmp = g_strdup_printf ("%s_ca_cert_chooser", prefix);
 	valid = validate_file_chooser (builder, tmp);
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENVPN_EDITOR_PLUGIN_ERROR,
+		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_CA);
 		return FALSE;
 	}
@@ -502,8 +444,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENVPN_EDITOR_PLUGIN_ERROR,
+		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_CERT);
 		return FALSE;
 	}
@@ -514,25 +456,30 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_PLUGIN_UI_ERROR,
-		             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENVPN_EDITOR_PLUGIN_ERROR,
+		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_KEY);
 		return FALSE;
 	}
 
 	/* Encrypted certificates require a password */
-	str = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
-	encrypted = is_encrypted (str);
-	g_free (str);
-	if (encrypted) {
-		tmp = g_strdup_printf ("%s_private_key_password_entry", prefix);
-		widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
-		g_free (tmp);
+	tmp = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
+	encrypted = is_encrypted (tmp);
+	g_free (tmp);
 
+	tmp = g_strdup_printf ("%s_private_key_password_entry", prefix);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
+	g_free (tmp);
+	pw_flags = nma_utils_menu_to_secret_flags (widget);
+	if (   pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED
+	    || pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)
+		secrets_required = FALSE;
+
+	if (encrypted && secrets_required) {
 		if (!gtk_entry_get_text_length (GTK_ENTRY (widget))) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_CERTPASS);
 			return FALSE;
 		}
@@ -558,16 +505,16 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_USERNAME);
 			return FALSE;
 		}
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
 		if (!validate_file_chooser (builder, "pw_ca_cert_chooser")) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_CA);
 			return FALSE;
 		}
@@ -575,16 +522,16 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_USERNAME);
 			return FALSE;
 		}
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		if (!validate_file_chooser (builder, "sk_key_chooser")) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_STATIC_KEY);
 			return FALSE;
 		}
@@ -593,8 +540,8 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_LOCAL_IP);
 			return FALSE;
 		}
@@ -603,8 +550,8 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_PLUGIN_UI_ERROR,
-			             OPENVPN_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+			             OPENVPN_EDITOR_PLUGIN_ERROR,
+			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_REMOTE_IP);
 			return FALSE;
 		}
@@ -619,7 +566,7 @@ update_from_filechooser (GtkBuilder *builder,
                          const char *key,
                          const char *prefix,
                          const char *widget_name,
-                         NMSettingVPN *s_vpn)
+                         NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	char *tmp, *filename;
@@ -641,7 +588,7 @@ update_from_filechooser (GtkBuilder *builder,
 }
 
 static void
-update_tls (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
+update_tls (GtkBuilder *builder, const char *prefix, NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	NMSettingSecretFlags pw_flags;
@@ -662,12 +609,12 @@ update_tls (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 	if (str && strlen (str))
 		nm_setting_vpn_add_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS, str);
 
-	pw_flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "flags"));
+	pw_flags = nma_utils_menu_to_secret_flags (widget);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_CERTPASS, pw_flags, NULL);
 }
 
 static void
-update_pw (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
+update_pw (GtkBuilder *builder, const char *prefix, NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	NMSettingSecretFlags pw_flags;
@@ -697,32 +644,14 @@ update_pw (GtkBuilder *builder, const char *prefix, NMSettingVPN *s_vpn)
 		nm_setting_vpn_add_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD, str);
 
 	/* Update password flags */
-	pw_flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "flags"));
-	pw_flags &= ~(NM_SETTING_SECRET_FLAG_NOT_SAVED | NM_SETTING_SECRET_FLAG_NOT_REQUIRED);
-
-	tmp = g_strdup_printf ("%s_pass_type_combo", prefix);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, tmp));
-	g_free (tmp);
-
-	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (widget))) {
-	case PW_TYPE_SAVE:
-		break;
-	case PW_TYPE_UNUSED:
-		pw_flags |= NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
-		break;
-	case PW_TYPE_ASK:
-	default:
-		pw_flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
-		break;
-	}
-
+	pw_flags = nma_utils_menu_to_secret_flags (widget);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_OPENVPN_KEY_PASSWORD, pw_flags, NULL);
 }
 
 gboolean
 auth_widget_update_connection (GtkBuilder *builder,
                                const char *contype,
-                               NMSettingVPN *s_vpn)
+                               NMSettingVpn *s_vpn)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -817,12 +746,13 @@ tls_default_filter (const GtkFileFilterInfo *filter_info, gpointer data)
 	if (!ext)
 		return FALSE;
 
-	if (pkcs_allowed && !strcmp (ext, ".p12") && is_pkcs12 (filter_info->filename)) {
+	if (pkcs_allowed && g_str_has_suffix (ext, ".p12") && is_pkcs12 (filter_info->filename)) {
 		g_free (ext);
 		return TRUE;
 	}
 
-	if (strcmp (ext, ".pem") && strcmp (ext, ".crt") && strcmp (ext, ".key") && strcmp (ext, ".cer")) {
+	if (!g_str_has_suffix (ext, ".pem") && !g_str_has_suffix (ext, ".crt") &&
+		!g_str_has_suffix (ext, ".key") && !g_str_has_suffix (ext, ".cer")) {
 		g_free (ext);
 		return FALSE;
 	}
@@ -906,7 +836,7 @@ sk_default_filter (const GtkFileFilterInfo *filter_info, gpointer data)
 	ext = g_ascii_strdown (p, -1);
 	if (!ext)
 		return FALSE;
-	if (strcmp (ext, ".key")) {
+	if (!g_str_has_suffix (ext, ".key")) {
 		g_free (ext);
 		return FALSE;
 	}
@@ -947,6 +877,7 @@ static const char *advanced_keys[] = {
 	NM_OPENVPN_KEY_PORT,
 	NM_OPENVPN_KEY_COMP_LZO,
 	NM_OPENVPN_KEY_MSSFIX,
+	NM_OPENVPN_KEY_FLOAT,
 	NM_OPENVPN_KEY_TUNNEL_MTU,
 	NM_OPENVPN_KEY_FRAGMENT_SIZE,
 	NM_OPENVPN_KEY_TAP_DEV,
@@ -966,7 +897,12 @@ static const char *advanced_keys[] = {
 	NM_OPENVPN_KEY_RENEG_SECONDS,
 	NM_OPENVPN_KEY_TLS_REMOTE,
 	NM_OPENVPN_KEY_REMOTE_RANDOM,
+	NM_OPENVPN_KEY_TUN_IPV6,
 	NM_OPENVPN_KEY_REMOTE_CERT_TLS,
+	NM_OPENVPN_KEY_NS_CERT_TYPE,
+	NM_OPENVPN_KEY_PING,
+	NM_OPENVPN_KEY_PING_EXIT,
+	NM_OPENVPN_KEY_PING_RESTART,
 	NULL
 };
 
@@ -989,12 +925,12 @@ advanced_dialog_new_hash_from_connection (NMConnection *connection,
                                           GError **error)
 {
 	GHashTable *hash;
-	NMSettingVPN *s_vpn;
-	const char *secret;
+	NMSettingVpn *s_vpn;
+	const char *secret, *flags;
 
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 	nm_setting_vpn_foreach_data_item (s_vpn, copy_values, hash);
 
 	/* HTTP Proxy password is special */
@@ -1004,6 +940,11 @@ advanced_dialog_new_hash_from_connection (NMConnection *connection,
 		                     g_strdup (NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD),
 		                     g_strdup (secret));
 	}
+	flags = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags");
+	if (flags)
+		g_hash_table_insert (hash,
+		                     g_strdup (NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags"),
+		                     g_strdup (flags));
 
 	return hash;
 }
@@ -1074,6 +1015,15 @@ populate_cipher_combo (GtkComboBox *box, const char *user_cipher)
 	                    TLS_CIPHER_COL_NAME, _("Default"),
 	                    TLS_CIPHER_COL_DEFAULT, TRUE, -1);
 
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+	                    TLS_CIPHER_COL_NAME, "none",
+	                    TLS_CIPHER_COL_DEFAULT, FALSE, -1);
+	if (g_strcmp0 (user_cipher, "none") == 0) {
+		gtk_combo_box_set_active_iter (box, &iter);
+		user_added = TRUE;
+	}
+
 	items = g_strsplit (tmp, "\n", 0);
 	g_free (tmp);
 
@@ -1090,12 +1040,15 @@ populate_cipher_combo (GtkComboBox *box, const char *user_cipher)
 		if (space)
 			*space = '\0';
 
+		if (strcmp (*item, "none") == 0)
+			continue;
+
 		if (strlen (*item)) {
 			gtk_list_store_append (store, &iter);
 			gtk_list_store_set (store, &iter,
 			                    TLS_CIPHER_COL_NAME, *item,
 			                    TLS_CIPHER_COL_DEFAULT, FALSE, -1);
-			if (user_cipher && !strcmp (*item, user_cipher)) {
+			if (!user_added && user_cipher && !strcmp (*item, user_cipher)) {
 				gtk_combo_box_set_active_iter (box, &iter);
 				user_added = TRUE;
 			}
@@ -1248,8 +1201,6 @@ tls_auth_toggled_cb (GtkWidget *widget, gpointer user_data)
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_checkbutton"));
 	use_auth = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_dir_help_label"));
-	gtk_widget_set_sensitive (widget, use_auth);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "direction_label"));
 	gtk_widget_set_sensitive (widget, use_auth);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_label"));
@@ -1260,6 +1211,53 @@ tls_auth_toggled_cb (GtkWidget *widget, gpointer user_data)
 	gtk_widget_set_sensitive (widget, use_auth);
 }
 
+static void
+ns_cert_type_toggled_cb (GtkWidget *widget, gpointer user_data)
+{
+	GtkBuilder *builder = (GtkBuilder *) user_data;
+	gboolean use_ns_cert_type = FALSE;
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_checkbutton"));
+	use_ns_cert_type = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_label"));
+	gtk_widget_set_sensitive (widget, use_ns_cert_type);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_combo"));
+	gtk_widget_set_sensitive (widget, use_ns_cert_type);
+}
+
+#define NS_CERT_TYPE_COL_NAME 0
+#define NS_CERT_TYPE_COL_VALUE 1
+
+static void
+populate_ns_cert_type_combo (GtkComboBox *box, const char *type)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+
+	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	gtk_combo_box_set_model (box, GTK_TREE_MODEL (store));
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+	                    NS_CERT_TYPE_COL_NAME, _("Server"),
+	                    NS_CERT_TYPE_COL_VALUE, NM_OPENVPN_NS_CERT_TYPE_SERVER,
+	                    -1);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+	                    NS_CERT_TYPE_COL_NAME, _("Client"),
+	                    NS_CERT_TYPE_COL_VALUE, NM_OPENVPN_NS_CERT_TYPE_CLIENT,
+	                    -1);
+
+	if (g_strcmp0 (type, NM_OPENVPN_NS_CERT_TYPE_CLIENT) == 0)
+		gtk_combo_box_set_active (box, 1);
+	else
+		gtk_combo_box_set_active (box, 0);
+
+	g_object_unref (store);
+}
+
 #define PROXY_TYPE_NONE  0
 #define PROXY_TYPE_HTTP  1
 #define PROXY_TYPE_SOCKS 2
@@ -1267,6 +1265,9 @@ tls_auth_toggled_cb (GtkWidget *widget, gpointer user_data)
 #define DEVICE_TYPE_IDX_TUN     0
 #define DEVICE_TYPE_IDX_TAP     1
 
+#define PING_EXIT    0
+#define PING_RESTART 1
+
 static void
 proxy_type_changed (GtkComboBox *combo, gpointer user_data)
 {
@@ -1406,6 +1407,19 @@ dev_checkbox_toggled_cb (GtkWidget *check, gpointer user_data)
 	device_name_changed_cb (GTK_ENTRY (entry), ok_button);
 }
 
+static void
+ping_exit_restart_checkbox_toggled_cb (GtkWidget *check, gpointer user_data)
+{
+	GtkBuilder *builder = (GtkBuilder *) user_data;
+	GtkWidget *combo, *spin;
+
+	combo = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_combo"));
+	spin = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+
+	checkbox_toggled_update_widget_cb (check, combo);
+	checkbox_toggled_update_widget_cb (check, spin);
+}
+
 #define TA_DIR_COL_NAME 0
 #define TA_DIR_COL_NUM 1
 
@@ -1420,7 +1434,8 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	const char *dev, *dev_type, *tap_dev;
 	GtkListStore *store;
 	GtkTreeIter iter;
-	guint32 active = PROXY_TYPE_NONE;
+	guint32 active;
+	guint32 pw_flags = NM_SETTING_SECRET_FLAG_NONE;
 	GError *error = NULL;
 
 	g_return_val_if_fail (hash != NULL, NULL);
@@ -1524,19 +1539,20 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 			tmp = strtol (value, NULL, 10);
 			if (errno != 0 || tmp < 0 || tmp > 65535)
 				tmp = 0;
-			widget = GTK_WIDGET (gtk_builder_get_object (builder, "proxy_password_entry"));
-			g_object_set_data (G_OBJECT (widget), "flags", GUINT_TO_POINTER ((guint32) tmp));
+			pw_flags = (guint32) tmp;
 		}
 	}
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "proxy_password_entry"));
+	nma_utils_setup_password_storage (widget, pw_flags, NULL, NULL,
+	                                  TRUE, FALSE);
 
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PROXY_TYPE);
+	active = PROXY_TYPE_NONE;
 	if (value) {
 		if (!strcmp (value, "http"))
 			active = PROXY_TYPE_HTTP;
 		else if (!strcmp (value, "socks"))
 			active = PROXY_TYPE_SOCKS;
-		else
-			active = PROXY_TYPE_NONE;
 	}
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
@@ -1610,7 +1626,7 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 
 		errno = 0;
 		tmp = strtol (value, NULL, 10);
-		if (errno == 0 && tmp > 0 && tmp < 65536) {
+		if (errno == 0 && tmp >= 0 && tmp < 65536) {
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
 			widget = GTK_WIDGET (gtk_builder_get_object (builder, "fragment_spinbutton"));
@@ -1625,12 +1641,6 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		gtk_widget_set_sensitive (widget, FALSE);
 	}
 
-	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_MSSFIX);
-	if (value && !strcmp (value, "yes")) {
-		widget = GTK_WIDGET (gtk_builder_get_object (builder, "mssfix_checkbutton"));
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-	}
-
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_COMP_LZO);
 	if (value && !strcmp (value, "yes")) {
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "lzo_checkbutton"));
@@ -1643,6 +1653,12 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	}
 
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_FLOAT);
+	if (value && !strcmp (value, "yes")) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "float_checkbutton"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+	}
+
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PROTO_TCP);
 	if (value && !strcmp (value, "yes")) {
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tcp_checkbutton"));
@@ -1688,6 +1704,12 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	}
 
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_TUN_IPV6);
+	if (value && !strcmp (value, "yes")) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tun_ipv6_checkbutton"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+	}
+
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "cipher_combo"));
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_CIPHER);
 	populate_cipher_combo (GTK_COMBO_BOX (widget), value);
@@ -1739,12 +1761,23 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_REMOTE_CERT_TLS);
 	populate_remote_cert_tls_combo (GTK_COMBO_BOX (widget), value);
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_checkbutton"));
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_NS_CERT_TYPE);
+	if (value && strlen (value))
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (ns_cert_type_toggled_cb), builder);
+	ns_cert_type_toggled_cb (widget, builder);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_combo"));
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_NS_CERT_TYPE);
+	populate_ns_cert_type_combo (GTK_COMBO_BOX (widget), value);
+
 	if (   !strcmp (contype, NM_OPENVPN_CONTYPE_TLS)
 	    || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)
 	    || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
 		int direction = -1;
 
-		active = -1;
+		active = 0;
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_checkbutton"));
 		value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_TA);
 		if (value && strlen (value))
@@ -1778,7 +1811,7 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 
 		gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
 		g_object_unref (store);
-		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 0 : active);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active);
 
 		value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_TA);
 		if (value && strlen (value)) {
@@ -1790,6 +1823,74 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 		gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2);
 	}
 
+	/* ping */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_checkbutton"));
+	spin = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (checkbox_toggled_update_widget_cb), spin);
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PING);
+	if (value && *value) {
+		long int tmp;
+
+		errno = 0;
+		tmp = strtol (value, NULL, 10);
+		if (errno == 0 && tmp > 0 && tmp < 65536) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gdouble) tmp);
+			gtk_widget_set_sensitive (widget, TRUE);
+		}
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 30.0);
+		gtk_widget_set_sensitive (widget, FALSE);
+	}
+
+	/* ping-exit / ping-restart */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_checkbutton"));
+	ping_exit_restart_checkbox_toggled_cb (widget, builder);
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (ping_exit_restart_checkbox_toggled_cb), builder);
+
+	combo = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_combo"));
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PING_EXIT);
+	if (value && *value)
+		active = PING_EXIT;
+	else {
+		value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_PING_RESTART);
+		active = PING_RESTART;
+	}
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("ping-exit"), -1);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("ping-restart"), -1);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
+	g_object_unref (store);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), active);
+
+	if (value && *value) {
+		long int tmp;
+
+		errno = 0;
+		tmp = strtol (value, NULL, 10);
+		if (errno == 0 && tmp > 0 && tmp < 65536) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (gdouble) tmp);
+			gtk_widget_set_sensitive (widget, TRUE);
+		}
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 30.0);
+		gtk_widget_set_sensitive (widget, FALSE);
+	}
+
 out:
 	g_free (ui_file);
 	return dialog;
@@ -1898,7 +1999,7 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 					                     g_strdup (value));
 				}
 
-				pw_flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "flags"));
+				pw_flags = nma_utils_menu_to_secret_flags (widget);
 				if (pw_flags != NM_SETTING_SECRET_FLAG_NONE) {
 					g_hash_table_insert (hash,
 					                     g_strdup (NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags"),
@@ -1916,6 +2017,10 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_MSSFIX), g_strdup ("yes"));
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "float_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_FLOAT), g_strdup ("yes"));
+
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tcp_checkbutton"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_PROTO_TCP), g_strdup ("yes"));
@@ -1940,6 +2045,10 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_REMOTE_RANDOM), g_strdup ("yes"));
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "tun_ipv6_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_TUN_IPV6), g_strdup ("yes"));
+
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "cipher_combo"));
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter)) {
@@ -2002,6 +2111,21 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 			}
 		}
 
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_checkbutton"));
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_combo"));
+			model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+			if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter)) {
+				char *type = NULL;
+
+				gtk_tree_model_get (model, &iter, NS_CERT_TYPE_COL_VALUE, &type, -1);
+				if (type)
+					g_hash_table_insert (hash,
+					                     g_strdup (NM_OPENVPN_KEY_NS_CERT_TYPE),
+					                     type);
+			}
+		}
+
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_checkbutton"));
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
 			char *filename;
@@ -2021,12 +2145,41 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 				gtk_tree_model_get (model, &iter, TA_DIR_COL_NUM, &direction, -1);
 				if (direction >= 0) {
 					g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_TA_DIR),
-					                     g_strdup_printf ("%d", direction));					
+					                     g_strdup_printf ("%d", direction));
 				}
 			}
 		}
 	}
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		int ping_val;
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_spinbutton"));
+		ping_val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
+
+		g_hash_table_insert (hash,
+		                     g_strdup (NM_OPENVPN_KEY_PING),
+		                     g_strdup_printf ("%d", ping_val));
+	}
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_checkbutton"));
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		int ping_exit_type, ping_val;
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_combo"));
+		ping_exit_type = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ping_exit_restart_spinbutton"));
+		ping_val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
+
+		g_hash_table_insert (hash,
+		                     ping_exit_type == PING_EXIT ?
+		                       g_strdup (NM_OPENVPN_KEY_PING_EXIT) :
+		                       g_strdup (NM_OPENVPN_KEY_PING_RESTART),
+		                     g_strdup_printf ("%d", ping_val));
+	}
+
 	return hash;
 }
 
